@@ -6,10 +6,13 @@ test_that("grpreg() reproduces simple linear regression", {
   group <- 1
   reg <- lm(y~X)$coef
   nlam=100
-  par(mfrow=c(3,2))
-  gMCP <- coef(fit <- grpreg(X, y, group, penalty="gMCP", nlambda=nlam, lambda.min=0))[,nlam]
+  par(mfcol=c(3,2))
+  gel <- coef(fit <- grpreg(X, y, group, penalty="gel", nlambda=nlam, lambda.min=0))[,nlam]
   plot(fit, main=fit$penalty)
-  expect_that(gMCP, equals(reg, tolerance=.01, check.attributes=FALSE))
+  expect_that(gel, equals(reg, tolerance=.01, check.attributes=FALSE))
+  cMCP <- coef(fit <- grpreg(X, y, group, penalty="cMCP", nlambda=nlam, lambda.min=0))[,nlam]
+  plot(fit, main=fit$penalty)
+  expect_that(cMCP, equals(reg, tolerance=.01, check.attributes=FALSE))
   bridge <- coef(fit <- gBridge(X, y, group, nlambda=nlam, lambda.min=0))[,1]
   plot(fit, main=fit$penalty)
   expect_that(bridge, equals(reg, tolerance=.01, check.attributes=FALSE))
@@ -32,20 +35,25 @@ test_that("grpreg() reproduces linear regression", {
   group <- rep(0:3,4:1)
   fit.mle <- lm(y~X)
   reg <- coef(fit.mle)
-  gMCP <- coef(fit <- grpreg(X, y, group, penalty="gMCP", lambda.min=0))[,100]
-  if (interactive()) {par(mfrow=c(3,2));plot(fit, main=fit$penalty)}
-  expect_that(gMCP, equals(reg, tolerance=.01, check.attributes=FALSE))
+  nlam=100
+  par(mfcol=c(3,2))
+  gel <- coef(fit <- grpreg(X, y, group, penalty="gel", nlambda=nlam, lambda.min=0))[,nlam]
+  plot(fit, main=fit$penalty)
+  expect_that(gel, equals(reg, tolerance=.01, check.attributes=FALSE))
+  cMCP <- coef(fit <- grpreg(X, y, group, penalty="cMCP", lambda.min=0))[,100]
+  plot(fit, main=fit$penalty)
+  expect_that(cMCP, equals(reg, tolerance=.01, check.attributes=FALSE))
   bridge <- coef(fit <- gBridge(X, y, group, lambda.min=0))[,1]
-  if (interactive()) {plot(fit, main=fit$penalty)}
+  plot(fit, main=fit$penalty)
   expect_that(bridge, equals(reg, tolerance=.01, check.attributes=FALSE))
   grLasso <- coef(fit <- grpreg(X, y, group, penalty="grLasso", lambda.min=0))[,100]
-  if (interactive()) {plot(fit, main=fit$penalty)}
+  plot(fit, main=fit$penalty)
   expect_that(grLasso, equals(reg, tolerance=.01, check.attributes=FALSE))
   grMCP <- coef(fit <- grpreg(X, y, group, penalty="grMCP", lambda.min=0))[,100]
-  if (interactive()) plot(fit, main=fit$penalty)
+  plot(fit, main=fit$penalty)
   expect_that(grMCP, equals(reg, tolerance=.01, check.attributes=FALSE))
   grSCAD <- coef(fit <- grpreg(X, y, group, penalty="grSCAD", lambda.min=0))[,100]
-  if (interactive()) plot(fit, main=fit$penalty)
+  plot(fit, main=fit$penalty)
   expect_that(grSCAD, equals(reg, tolerance=.01, check.attributes=FALSE))
   expect_that(predict(fit, X)[,100], equals(predict(fit.mle), tolerance=.001, check.attributes=FALSE))
 })
@@ -58,46 +66,53 @@ test_that("grpreg() reproduces simple logistic regression", {
   group <- 1
   reg <- glm(y~X, family="binomial")$coef
   nlam <- 100
-  gMCP <- coef(fit <- grpreg(X, y, group, penalty="gMCP", nlambda=nlam, lambda.min=0, family="binomial", gamma=12))[,nlam]
-  if (interactive()) {par(mfrow=c(3,2));plot(fit, main=fit$penalty)}
-  expect_that(gMCP, equals(reg, tolerance=.01, check.attributes=FALSE))
+  par(mfcol=c(3,2))
+  gel <- coef(fit <- grpreg(X, y, group, penalty="gel", nlambda=nlam, lambda.min=0, family="binomial", gamma=12))[,nlam]
+  plot(fit, main=fit$penalty)
+  expect_that(gel, equals(reg, tolerance=.01, check.attributes=FALSE))
+  cMCP <- coef(fit <- grpreg(X, y, group, penalty="cMCP", nlambda=nlam, lambda.min=0, family="binomial", gamma=12))[,nlam]
+  plot(fit, main=fit$penalty)
+  expect_that(cMCP, equals(reg, tolerance=.01, check.attributes=FALSE))
   bridge <- coef(fit <- gBridge(X, y, group, lambda.min=0, nlambda=nlam, family="binomial"))[,1]
-  if (interactive()) {plot(fit, main=fit$penalty)}
+  plot(fit, main=fit$penalty)
   expect_that(bridge, equals(reg, tolerance=.01, check.attributes=FALSE))
   grLasso <- coef(fit <- grpreg(X, y, group, penalty="grLasso", nlambda=nlam, lambda.min=0, family="binomial"))[,nlam]
-  if (interactive()) {plot(fit, main=fit$penalty)}
+  plot(fit, main=fit$penalty)
   expect_that(grLasso, equals(reg, tolerance=.01, check.attributes=FALSE))
   grMCP <- coef(fit <- grpreg(X, y, group, penalty="grMCP", nlambda=nlam, lambda.min=0, family="binomial"))[,nlam]
-  if (interactive()) {plot(fit, main=fit$penalty)}
+  plot(fit, main=fit$penalty)
   expect_that(grMCP, equals(reg, tolerance=.01, check.attributes=FALSE))
   grSCAD <- coef(fit <- grpreg(X, y, group, penalty="grSCAD", nlambda=nlam, lambda.min=0, family="binomial"))[,nlam]
-  if (interactive()) {plot(fit, main=fit$penalty)}
+  plot(fit, main=fit$penalty)
   expect_that(grSCAD, equals(reg, tolerance=.01, check.attributes=FALSE))
 })
 
 test_that("grpreg() reproduces logistic regression", {
   n <- 50
   group <- rep(0:3,1:4)
-  ## group <- rep(1:3,rep(3,3))
   p <- length(group)
   X <- matrix(rnorm(n*p),ncol=p)
   y <- runif(n) > .5
   fit.mle <- glm(y~X, family="binomial")
   reg <- coef(fit.mle)
-  gMCP <- coef(fit <- grpreg(X, y, group, penalty="gMCP", family="binomial"))[,100]
-  if (interactive()) {par(mfrow=c(3,2));plot(fit, main=fit$penalty)}
-  expect_that(gMCP, equals(reg, tolerance=.01, check.attributes=FALSE))
+  par(mfcol=c(3,2))
+  gel <- coef(fit <- grpreg(X, y, group, penalty="gel", family="binomial"))[,100]
+  plot(fit, main=fit$penalty)
+  expect_that(gel, equals(reg, tolerance=.01, check.attributes=FALSE))
+  cMCP <- coef(fit <- grpreg(X, y, group, penalty="cMCP", family="binomial"))[,100]
+  plot(fit, main=fit$penalty)
+  expect_that(cMCP, equals(reg, tolerance=.01, check.attributes=FALSE))
   bridge <- coef(fit <- gBridge(X, y, group, family="binomial"))[,1]
-  if (interactive()) plot(fit, main=fit$penalty)
+  plot(fit, main=fit$penalty)
   expect_that(bridge, equals(reg, tolerance=.01, check.attributes=FALSE))
   grLasso <- coef(fit <- grpreg(X, y, group, penalty="grLasso", family="binomial"))[,100]
-  if (interactive()) plot(fit, main=fit$penalty)
+  plot(fit, main=fit$penalty)
   expect_that(grLasso, equals(reg, tolerance=.01, check.attributes=FALSE))
   grMCP <- coef(fit <- grpreg(X, y, group, penalty="grMCP", family="binomial", gamma=2))[,100]
-  if (interactive()) plot(fit, main=fit$penalty)
+  plot(fit, main=fit$penalty)
   expect_that(grMCP, equals(reg, tolerance=.01, check.attributes=FALSE))
   grSCAD <- coef(fit <- grpreg(X, y, group, penalty="grSCAD", family="binomial", gamma=2))[,100]
-  if (interactive()) plot(fit, main=fit$penalty)
+  plot(fit, main=fit$penalty)
   expect_that(grSCAD, equals(reg, tolerance=.01, check.attributes=FALSE))
   expect_that(predict(fit, X)[,100], equals(predict(fit.mle), tolerance=.001, check.attributes=FALSE))
   expect_that(predict(fit, X, type="response")[,100], equals(predict(fit.mle, type="response"), tolerance=.001, check.attributes=FALSE))
