@@ -6,8 +6,7 @@
 #include <R_ext/Applic.h>
 
 // Check for convergence of beta[l]
-static int checkConvergence(double *beta, double *beta_old, double eps, int l, int J)
-{
+static int checkConvergence(double *beta, double *beta_old, double eps, int l, int J) {
   int j;
   int converged = 1;
   for (j=0; j < J; j++) {
@@ -28,8 +27,7 @@ static int checkConvergence(double *beta, double *beta_old, double eps, int l, i
 }
 
 // Cross product of the jth column of x with y
-static double crossprod(double *x, double *y, int n, int j)
-{
+static double crossprod(double *x, double *y, int n, int j) {
   double val = 0;
   int nn = n*j;
   for (int i=0; i<n; i++) val += x[nn+i] * y[i];
@@ -37,24 +35,28 @@ static double crossprod(double *x, double *y, int n, int j)
 }
 
 // Mean of x
-static double mean(double *x, int n)
-{
+static double mean(double *x, int n) {
   double val = 0;
   for (int i=0; i<n; i++) val += x[i];
   return(val/n);
 }
 
+// Sum of x
+static double sum(int *x, int n) {
+  double val = 0;
+  for (int i=0; i<n; i++) val += x[i];
+  return(val);
+}
+
 // Gaussian loss (squared error loss)
-static double gLoss(double *r, int n)
-{
+static double gLoss(double *r, int n) {
   double l = 0;
   for (int i=0;i<n;i++) l = l + pow(r[i],2);
   return(l);
 }
 
 // Euclidean norm
-static double norm(double *x, int p)
-{
+static double norm(double *x, int p) {
   double x_norm = 0;
   for (int j=0; j<p; j++) x_norm = x_norm + pow(x[j],2);
   x_norm = sqrt(x_norm);
@@ -62,16 +64,14 @@ static double norm(double *x, int p)
 }
 
 // Soft-thresholding operator
-static double S(double z, double l)
-{
+static double S(double z, double l) {
   if (z > l) return(z-l);
   if (z < -l) return(z+l);
   return(0);
 }
 
 // Firm-thresholding operator
-static double F(double z, double l1, double l2, double gamma)
-{
+static double F(double z, double l1, double l2, double gamma) {
   double s=0;
   if (z > 0) s = 1;
   else if (z < 0) s = -1;
@@ -81,8 +81,7 @@ static double F(double z, double l1, double l2, double gamma)
 }
 
 // SCAD-modified firm-thresholding operator
-static double Fs(double z, double l1, double l2, double gamma)
-{
+static double Fs(double z, double l1, double l2, double gamma) {
   double s=0;
   if (z > 0) s = 1;
   else if (z < 0) s = -1;
@@ -93,24 +92,21 @@ static double Fs(double z, double l1, double l2, double gamma)
 }
 
 // MCP penalty
-static double MCP(double theta, double l, double a)
-{
+static double MCP(double theta, double l, double a) {
   theta = fabs(theta);
   if (theta <= a*l) return(l*theta - pow(theta,2)/(2*a));
   else return(a*pow(l,2)/2);
 }
 
 // MCP penalization rate
-static double dMCP(double theta, double l, double a)
-{
+static double dMCP(double theta, double l, double a) {
   theta = fabs(theta);
   if (theta < a*l) return(l-theta/a);
   else return(0);
 }
 
 // Group descent update
-static void gd_gaussian(double *beta, double *x, double *r, int g, int *K1, int *active, int n, int l, int p, char *penalty, double lam1, double lam2, double gamma, double *df, double *beta_old)
-{
+static void gd_gaussian(double *beta, double *x, double *r, int g, int *K1, int n, int l, int p, char *penalty, double lam1, double lam2, double gamma, double *df, double *beta_old) {
   // Calculate z
   int K = K1[g+1] - K1[g];
   double *z = Calloc(K, double);
@@ -122,14 +118,13 @@ static void gd_gaussian(double *beta, double *x, double *r, int g, int *K1, int
   if (strcmp(penalty, "grLasso")==0) len = S(z_norm, lam1) / (1+lam2);
   if (strcmp(penalty, "grMCP")==0) len = F(z_norm, lam1, lam2, gamma);
   if (strcmp(penalty, "grSCAD")==0) len = Fs(z_norm, lam1, lam2, gamma);
-  if (len != 0 | active[g]) {
+  if (len != 0 | beta_old[K1[g]] != 0) {
     // If necessary, update beta and r
     for (int j=K1[g]; j<K1[g+1]; j++) {
       beta[l*p+j] = len * z[j-K1[g]] / z_norm;
       double shift = beta[l*p+j]-beta_old[j];
       for (int i=0; i<n; i++) r[i] -= x[n*j+i] * shift;
     }
-    if (len > 0) active[g] = 1; else active[g] = 0;
   }
 
   // Update df
@@ -138,8 +133,7 @@ static void gd_gaussian(double *beta, double *x, double *r, int g, int *K1, int
 }
 
 // Group descent update -- binomial
-static void gd_binomial(double *beta, double *x, double *r, int g, int *K1, int *active, int n, int l, int p, char *penalty, double lam1, double lam2, double gamma, double *df, double *beta_old)
-{
+static void gd_binomial(double *beta, double *x, double *r, int g, int *K1, int n, int l, int p, char *penalty, double lam1, double lam2, double gamma, double *df, double *beta_old) {
   // Calculate z
   int K = K1[g+1] - K1[g];
   double *z = Calloc(K, double);
@@ -152,14 +146,13 @@ static void gd_binomial(double *beta, double *x, double *r, int g, int *K1, int
   if (strcmp(penalty, "grLasso")==0) len = S(v * z_norm, lam1) / (v * (1 + lam2));
   if (strcmp(penalty, "grMCP")==0) len = F(v * z_norm, lam1, lam2, gamma) / v;
   if (strcmp(penalty, "grSCAD")==0) len = Fs(v * z_norm, lam1, lam2, gamma) / v;
-  if (len != 0 | active[g]) {
+  if (len != 0 | beta_old[K1[g]] != 0) {
     // If necessary, update beta and r
     for (int j=K1[g]; j<K1[g+1]; j++) {
       beta[l*p+j] = len * z[j-K1[g]] / z_norm;
       double shift = beta[l*p+j]-beta_old[j];
       for (int i=0; i<n; i++) r[i] -= x[n*j+i] * shift;
     }
-    if (len > 0) active[g] = 1; else active[g] = 0;
   }
 
   // Update df
@@ -173,18 +166,17 @@ static void gLCD_gaussian(double *beta, char *penalty, double *x, double *r, int
   // Make initial local approximation
   int K = K1[g+1] - K1[g];
   double sG = 0; // Sum of inner penalties for group
-  if (strcmp(penalty, "geLasso")==0) for (int j=K1[g]; j<K1[g+1]; j++) sG = sG + fabs(beta_old[j]);
-  if (strcmp(penalty, "geMCP")==0) for (int j=K1[g]; j<K1[g+1]; j++) sG = sG + MCP(beta_old[j], lam1, gamma);
-  if (strcmp(penalty, "gMCP")==0) for (int j=K1[g]; j<K1[g+1]; j++) sG = sG + MCP(beta_old[j], lam1, gamma);
+  if (strcmp(penalty, "gel")==0) for (int j=K1[g]; j<K1[g+1]; j++) sG = sG + fabs(beta_old[j]);
+  if (strcmp(penalty, "cMCP")==0) for (int j=K1[g]; j<K1[g+1]; j++) sG = sG + MCP(beta_old[j], lam1, gamma);
   if (strcmp(penalty, "gBridge")==0) {
-      for (int j=K1[g]; j<K1[g+1]; j++) sG = sG + fabs(beta_old[j]);
-      if (sG==0) return;
-      if (sG < delta) {
-	for (int j=K1[g]; j<K1[g+1]; j++) {
-	  beta[l*p+j] = 0;
-	  for (int i=0; i<n; i++) r[i] = r[i] - (beta[l*p+j] - beta_old[j]) * x[n*j+i];
-	}
-	return;
+    for (int j=K1[g]; j<K1[g+1]; j++) sG = sG + fabs(beta_old[j]);
+    if (sG==0) return;
+    if (sG < delta) {
+      for (int j=K1[g]; j<K1[g+1]; j++) {
+	beta[l*p+j] = 0;
+	for (int i=0; i<n; i++) r[i] = r[i] - (beta[l*p+j] - beta_old[j]) * x[n*j+i];
+      }
+      return;
     }
   }
 
@@ -199,9 +191,8 @@ static void gLCD_gaussian(double *beta, char *penalty, double *x, double *r, int
     // Calculate ljk
     double ljk=0;
     if (lam1 != 0) {
-      if (strcmp(penalty, "gMCP")==0) ljk = dMCP(sG, lam1, (K*gamma*pow(lam1,2))/(2*lam1)) * dMCP(beta[l*p+j], lam1, gamma);
-      if (strcmp(penalty, "geLasso")==0) ljk = lam1*exp(-tau/lam1*sG);
-      if (strcmp(penalty, "geMCP")==0) ljk = exp(-tau/pow(lam1,2)*sG)*dMCP(beta[j],lam1,gamma);
+      if (strcmp(penalty, "cMCP")==0) ljk = dMCP(sG, lam1, (K*gamma*pow(lam1,2))/(2*lam1)) * dMCP(beta[l*p+j], lam1, gamma);
+      if (strcmp(penalty, "gel")==0) ljk = lam1*exp(-tau/lam1*sG);
       if (strcmp(penalty,"gBridge")==0) ljk = lam1 * gamma * pow(sG, gamma-1);
     }
 
@@ -212,9 +203,8 @@ static void gLCD_gaussian(double *beta, char *penalty, double *x, double *r, int
     if (beta[l*p+j] != beta_old[j]) {
       for (int i=0; i<n; i++) r[i] = r[i] - (beta[l*p+j] - beta_old[j]) * x[n*j+i];
       if (strcmp(penalty, "gBridge")==0) sG = sG + fabs(beta[l*p+j]) - fabs(beta_old[j]);
-      if (strcmp(penalty, "geLasso")==0) sG = sG + fabs(beta[l*p+j]) - fabs(beta_old[j]);
-      if (strcmp(penalty, "geMCP")==0) sG = sG + MCP(beta[l*p+j], lam1, gamma) - MCP(beta_old[j], lam1, gamma);
-      if (strcmp(penalty, "gMCP")==0) sG = sG + MCP(beta[l*p+j], lam1, gamma) - MCP(beta_old[j], lam1, gamma);
+      if (strcmp(penalty, "gel")==0) sG = sG + fabs(beta[l*p+j]) - fabs(beta_old[j]);
+      if (strcmp(penalty, "cMCP")==0) sG = sG + MCP(beta[l*p+j], lam1, gamma) - MCP(beta_old[j], lam1, gamma);
     }
 
     // Update df
@@ -235,18 +225,17 @@ static void gLCD_binomial(double *beta, char *penalty, double *x, double *r, int
 
   // Make initial local approximation
   double sG = 0; // Sum of inner penalties for group
-  if (strcmp(penalty, "geLasso")==0) for (int j=K1[g]; j<K1[g+1]; j++) sG = sG + fabs(beta_old[j]);
-  if (strcmp(penalty, "geMCP")==0) for (int j=K1[g]; j<K1[g+1]; j++) sG = sG + MCP(beta_old[j], lam1, gamma);
-  if (strcmp(penalty, "gMCP")==0) for (int j=K1[g]; j<K1[g+1]; j++) sG = sG + MCP(beta_old[j], lam1, gamma);
+  if (strcmp(penalty, "gel")==0) for (int j=K1[g]; j<K1[g+1]; j++) sG = sG + fabs(beta_old[j]);
+  if (strcmp(penalty, "cMCP")==0) for (int j=K1[g]; j<K1[g+1]; j++) sG = sG + MCP(beta_old[j], lam1, gamma);
   if (strcmp(penalty, "gBridge")==0) {
-      for (int j=K1[g]; j<K1[g+1]; j++) sG = sG + fabs(beta_old[j]);
-      if (sG==0) return;
-      if (sG < delta) {
-	for (int j=K1[g]; j<K1[g+1]; j++) {
-	  beta[l*p+j] = 0;
-	  for (int i=0; i<n; i++) r[i] = r[i] - (beta[l*p+j] - beta_old[j]) * x[n*j+i];
-	}
-	return;
+    for (int j=K1[g]; j<K1[g+1]; j++) sG = sG + fabs(beta_old[j]);
+    if (sG==0) return;
+    if (sG < delta) {
+      for (int j=K1[g]; j<K1[g+1]; j++) {
+	beta[l*p+j] = 0;
+	for (int i=0; i<n; i++) r[i] = r[i] - (beta[l*p+j] - beta_old[j]) * x[n*j+i];
+      }
+      return;
     }
   }
 
@@ -261,10 +250,9 @@ static void gLCD_binomial(double *beta, char *penalty, double *x, double *r, int
     // Calculate ljk
     double ljk=0;
     if (lam1 != 0) {
-      if (strcmp(penalty, "gMCP")==0) ljk = dMCP(sG, lam1, (K*gamma*pow(lam1,2))/(2*lam1)) * dMCP(beta[l*p+j], lam1, gamma);
-      if (strcmp(penalty, "geLasso")==0) ljk = lam1*exp(-tau/lam1*sG);
-      if (strcmp(penalty, "geMCP")==0) ljk = exp(-tau/pow(lam1,2)*sG)*dMCP(beta[j],lam1,gamma);
-      if (strcmp(penalty,"gBridge")==0) ljk = lam1 * gamma * pow(sG, gamma-1);
+      if (strcmp(penalty, "cMCP")==0) ljk = dMCP(sG, lam1, (K*gamma*pow(lam1,2))/(2*lam1)) * dMCP(beta[l*p+j], lam1, gamma);
+      if (strcmp(penalty, "gel")==0) ljk = lam1*exp(-tau*v[j-K1[g]]/lam1*sG);
+      if (strcmp(penalty, "gBridge")==0) ljk = lam1 * gamma * pow(sG, gamma-1);
     }
 
     // Update beta
@@ -275,9 +263,8 @@ static void gLCD_binomial(double *beta, char *penalty, double *x, double *r, int
     if (beta[l*p+j] != beta_old[j]) {
       for (int i=0; i<n; i++) r[i] = r[i] - (beta[l*p+j] - beta_old[j]) * x[n*j+i];
       if (strcmp(penalty, "gBridge")==0) sG = sG + fabs(beta[l*p+j]) - fabs(beta_old[j]);
-      if (strcmp(penalty, "geLasso")==0) sG = sG + fabs(beta[l*p+j]) - fabs(beta_old[j]);
-      if (strcmp(penalty, "geMCP")==0) sG = sG + MCP(beta[l*p+j], lam1, gamma) - MCP(beta_old[j], lam1, gamma);
-      if (strcmp(penalty, "gMCP")==0) sG = sG + MCP(beta[l*p+j], lam1, gamma) - MCP(beta_old[j], lam1, gamma);
+      if (strcmp(penalty, "gel")==0) sG = sG + fabs(beta[l*p+j]) - fabs(beta_old[j]);
+      if (strcmp(penalty, "cMCP")==0) sG = sG + MCP(beta[l*p+j], lam1, gamma) - MCP(beta_old[j], lam1, gamma);
     }
 
     // Update df
@@ -286,13 +273,15 @@ static void gLCD_binomial(double *beta, char *penalty, double *x, double *r, int
   Free(v);
 }
 
-static void gpPathFit_gaussian(double *beta, int *iter, double *df, double *loss, double *x, double *y, int *n_, int *p_, char **penalty_, int *J_, int *K1, int *K0_, double *lam1, double *lam2, int *L_, double *eps_, double *delta_, int *max_iter_, double *gamma_, double *tau_, int *dfmax_, double *group_multiplier, int *user_)
+static void gpPathFit_gaussian(double *beta, int *iter, double *df, double *loss, double *x, double *y, int *n_, int *p_, char **penalty_, int *J_, int *K1, int *K0_, double *lam1, double *lam2, int *L_, double *eps_, double *delta_, int *max_iter_, double *gamma_, double *tau_, int *dfmax_, int *gmax_, double *group_multiplier, int *user_)
 {
   // Initialization of variables
-  int n=n_[0]; int p=p_[0]; char *penalty=penalty_[0]; int J=J_[0]; int K0=K0_[0]; int L=L_[0]; int max_iter=max_iter_[0]; double eps=eps_[0]; double delta=delta_[0]; double gamma=gamma_[0]; double tau=tau_[0]; int dfmax=dfmax_[0]; int user=user_[0];
-  double *r = Calloc(n, double);
+  int n=n_[0]; int p=p_[0]; char *penalty=penalty_[0]; int J=J_[0]; int K0=K0_[0]; int L=L_[0]; int max_iter=max_iter_[0]; double eps=eps_[0]; double delta=delta_[0]; double gamma=gamma_[0]; double tau=tau_[0]; int dfmax=dfmax_[0]; int gmax=gmax_[0]; int user=user_[0];
   double *beta_old = Calloc(p, double);
+  double *r = Calloc(n, double);
   for (int i=0; i<n; i++) r[i] = y[i];
+  int *active = Calloc(p, int);
+  for (int j=0; j<p; j++) active[j] = 0;
   if (strcmp(penalty, "gBridge")==0) {
     for (int j=0; j<p; j++) {
       beta_old[j] = crossprod(x, r, n, j)/n;
@@ -303,21 +292,30 @@ static void gpPathFit_gaussian(double *beta, int *iter, double *df, double *loss
   // Path
   for (int l=0; l<L; l++) {
     if (l != 0) for (int j=0; j<p; j++) beta_old[j] = beta[(l-1)*p+j];
+
+    // Check dfmax, gmax
+    int ng = 0;
+    int nv = 0;
+    for (int g=0; g<J; g++) {
+      int nv_old = nv;
+      for (int j=K1[g]; j<K1[g+1]; j++) {
+	if (beta_old[j] != 0) nv++;
+      }
+      if (nv != nv_old) ng++;
+    }
+    if (ng > gmax | nv > dfmax) {
+      for (int ll=l; ll<L; ll++) {
+	for (int j=0; j<p; j++) beta[ll*p+j] = R_NaReal;
+      }
+      Free(beta_old);
+      Free(r);
+      Free(active);
+      return;
+    }
+
     while (iter[l] < max_iter) {
       int converged = 0;
       iter[l]++;
-
-      // Check dfmax
-      int a = 0;
-      for (int j=0; j<p; j++) if (beta[l*p+j]!=0) a++;
-      if (a > dfmax) {
-	for (int ll=l; ll<L; ll++) {
-	  for (int j=0; j<p; j++) beta[ll*p+j] = R_NaReal;
-	}
-	Free(beta_old);
-	Free(r);
-	return;
-      }
       df[l] = 0;
 
       // Update unpenalized covariates
@@ -343,16 +341,19 @@ static void gpPathFit_gaussian(double *beta, int *iter, double *df, double *loss
     }
   }
   Free(beta_old);
+  Free(active);
   Free(r);
 }
 
-static void gpPathFit_binomial(double *beta0, double *beta, int *iter, double *df, double *Dev, double *x, double *y, int *n_, int *p_, char **penalty_, int *J_, int *K1, int *K0_, double *lam1, double *lam2, int *L_, double *eps_, double *delta_, int *max_iter_, double *gamma_, double *tau_, double *group_multiplier, int *dfmax_, int *warn_, int *user_)
+static void gpPathFit_binomial(double *beta0, double *beta, int *iter, double *df, double *Dev, double *x, double *y, int *n_, int *p_, char **penalty_, int *J_, int *K1, int *K0_, double *lam1, double *lam2, int *L_, double *eps_, double *delta_, int *max_iter_, double *gamma_, double *tau_, double *group_multiplier, int *dfmax_, int *gmax_, int *warn_, int *user_)
 {
-  int n=n_[0]; int p=p_[0]; char *penalty=penalty_[0]; int J=J_[0]; int K0=K0_[0]; int L=L_[0]; int max_iter=max_iter_[0]; double eps=eps_[0]; double delta=delta_[0]; double gamma=gamma_[0]; double tau=tau_[0]; int dfmax=dfmax_[0]; int warn = warn_[0]; int user = user_[0];
+  int n=n_[0]; int p=p_[0]; char *penalty=penalty_[0]; int J=J_[0]; int K0=K0_[0]; int L=L_[0]; int max_iter=max_iter_[0]; double eps=eps_[0]; double delta=delta_[0]; double gamma=gamma_[0]; double tau=tau_[0]; int dfmax=dfmax_[0]; int gmax=gmax_[0]; int warn = warn_[0]; int user = user_[0];
   double beta0_old=0;
   double *r = Calloc(n, double);
   double *w = Calloc(n, double);
   double *beta_old = Calloc(p, double);
+  int *active = Calloc(p, int);
+  for (int j=0; j<p; j++) active[j] = 0;
 
   // Initialization
   double ybar = mean(y,n);
@@ -374,22 +375,31 @@ static void gpPathFit_binomial(double *beta0, double *beta, int *iter, double *d
       beta0_old = beta0[l-1];
       for (int j=0; j<p; j++) beta_old[j] = beta[(l-1)*p+j];
     }
+
+    // Check dfmax, gmax
+    int ng = 0;
+    int nv = 0;
+    for (int g=0; g<J; g++) {
+      int nv_old = nv;
+      for (int j=K1[g]; j<K1[g+1]; j++) {
+	if (beta_old[j] != 0) nv++;
+      }
+      if (nv != nv_old) ng++;
+    }
+    if (ng > gmax | nv > dfmax) {
+      for (int ll=l; ll<L; ll++) {
+	for (int j=0; j<p; j++) beta[ll*p+j] = R_NaReal;
+      }
+      Free(beta_old);
+      Free(r);
+      Free(active);
+      return;
+    }
+
     while (iter[l] < max_iter) {
       int converged = 0;
       iter[l] = iter[l] + 1;
 
-      // Check dfmax
-      int a = 0;
-      for (int j=0; j<p; j++) if (beta[l*p+j]!=0) a++;
-      if (a > dfmax) {
-	for (int ll=l; ll<L; ll++) {
-	  for (int j=0; j<p; j++) beta[ll*p+j] = R_NaReal;
-	}
-	Free(beta_old);
-	Free(r);
-	return;
-      }
-
       // Approximate L
       Dev[l] = 0;
       for (int i=0; i<n; i++) {
@@ -418,6 +428,7 @@ static void gpPathFit_binomial(double *beta0, double *beta, int *iter, double *d
 	}
 	Free(beta_old);
 	Free(w);
+	Free(active);
 	Free(r);
 	return;
       }
@@ -462,15 +473,17 @@ static void gpPathFit_binomial(double *beta0, double *beta, int *iter, double *d
   }
   Free(beta_old);
   Free(w);
+  Free(active);
   Free(r);
 }
 
-static void grPathFit_gaussian(double *beta, int *iter, double *df, double *loss, double *x, double *y, int *n_, int *p_, char **penalty_, int *J_, int *K1, int *K0_, double *lam1, double *lam2, int *L_, double *eps_, int *max_iter_, double *gamma_, double *group_multiplier, int *dfmax_, int *user_)
+static void grPathFit_gaussian(double *beta, int *iter, double *df, double *loss, double *x, double *y, int *n_, int *p_, char **penalty_, int *J_, int *K1, int *K0_, double *lam1, double *lam2, int *L_, double *eps_, int *max_iter_, double *gamma_, double *group_multiplier, int *dfmax_, int *gmax_, int *user_)
 {
-  int n=n_[0]; int p=p_[0]; char *penalty=penalty_[0]; int J=J_[0]; int K0=K0_[0]; int L=L_[0]; int max_iter=max_iter_[0]; double eps=eps_[0]; double gamma=gamma_[0]; int dfmax=dfmax_[0]; int user = user_[0];
+  int n=n_[0]; int p=p_[0]; char *penalty=penalty_[0]; int J=J_[0]; int K0=K0_[0]; int L=L_[0]; int max_iter=max_iter_[0]; double eps=eps_[0]; double gamma=gamma_[0]; int dfmax=dfmax_[0]; int gmax=gmax_[0]; int user = user_[0];
   double *r = Calloc(n, double);
   double *beta_old = Calloc(p, double);
   int *active = Calloc(J, int);
+  for (int g=0; g<J; g++) active[g] = 0;
 
   // Initialization
   for (int i=0; i<n; i++) r[i] = y[i];
@@ -479,22 +492,30 @@ static void grPathFit_gaussian(double *beta, int *iter, double *df, double *loss
   for (int l=0; l<L; l++) {
     if (l != 0) for (int j=0; j<p; j++) beta_old[j] = beta[(l-1)*p+j];
     else for (int j=0; j<p; j++) beta_old[j] = 0;
+
+    // Check dfmax, gmax
+    int ng = 0;
+    int nv = 0;
+    for (int g=0; g<J; g++) {
+      if (beta_old[K1[g]] != 0) {
+	ng++;
+	nv = nv + (K1[g+1]-K1[g]);
+      }
+    }
+    if (ng > gmax | nv > dfmax) {
+      for (int ll=l; ll<L; ll++) {
+	for (int j=0; j<p; j++) beta[ll*p+j] = R_NaReal;
+      }
+      Free(beta_old);
+      Free(r);
+      Free(active);
+      return;
+    }
+
     while (iter[l] < max_iter) {
       int converged = 0;
       iter[l]++;
 
-      // Check dfmax
-      int a = 0;
-      for (int j=0; j<J; j++) a += (beta_old[j] !=0);
-      if (a > dfmax) {
-	for (int ll=l; ll<L; ll++) {
-	  for (int j=0; j<p; j++) beta[ll*p+j] = R_NaReal;
-	}
-	Free(beta_old);
-	Free(r);
-	Free(active);
-	return;
-      }
       df[l] = 0;
 
       // Update unpenalized covariates
@@ -508,7 +529,7 @@ static void grPathFit_gaussian(double *beta, int *iter, double *df, double *loss
       // Update penalized groups
       for (int g=0; g<J; g++) {
 	//Rprintf("g=%d  K1[g]=%d  K1[g+1]=%d  gm[g] = %f\n",g,K1[g],K1[g+1],group_multiplier[g]);
-	if (user | l!=0) gd_gaussian(beta, x, r, g, K1, active, n, l, p, penalty, lam1[l]*group_multiplier[g], lam2[l], gamma, df, beta_old);
+	if (user | l!=0) gd_gaussian(beta, x, r, g, K1, n, l, p, penalty, lam1[l]*group_multiplier[g], lam2[l], gamma, df, beta_old);
       }
 
       // Check convergence
@@ -525,13 +546,14 @@ static void grPathFit_gaussian(double *beta, int *iter, double *df, double *loss
   Free(r);
 }
 
-static void grPathFit_binomial(double *beta0, double *beta, int *iter, double *df, double *Dev, double *x, double *y, int *n_, int *p_, char **penalty_, int *J_, int *K1, int *K0_, double *lam1, double *lam2, int *L_, double *eps_, int *max_iter_, double *gamma_, double *group_multiplier, int *dfmax_, int *warn_, int *user_)
+static void grPathFit_binomial(double *beta0, double *beta, int *iter, double *df, double *Dev, double *x, double *y, int *n_, int *p_, char **penalty_, int *J_, int *K1, int *K0_, double *lam1, double *lam2, int *L_, double *eps_, int *max_iter_, double *gamma_, double *group_multiplier, int *dfmax_, int *gmax_, int *warn_, int *user_)
 {
-  int n=n_[0]; int p=p_[0]; char *penalty=penalty_[0]; int J=J_[0]; int K0=K0_[0]; int L=L_[0]; int max_iter=max_iter_[0]; double eps=eps_[0]; double gamma=gamma_[0]; int dfmax=dfmax_[0]; int warn=warn_[0]; int user=user_[0];
+  int n=n_[0]; int p=p_[0]; char *penalty=penalty_[0]; int J=J_[0]; int K0=K0_[0]; int L=L_[0]; int max_iter=max_iter_[0]; double eps=eps_[0]; double gamma=gamma_[0]; int dfmax=dfmax_[0]; int gmax=gmax_[0]; int warn=warn_[0]; int user=user_[0];
   double beta0_old=0;
   double *r = Calloc(n, double);
   double *beta_old = Calloc(p, double);
   int *active = Calloc(J, int);
+  for (int g=0; g<J; g++) active[g] = 0;
 
   // Initialization
   double ybar = mean(y,n);
@@ -546,30 +568,37 @@ static void grPathFit_binomial(double *beta0, double *beta, int *iter, double *d
       beta0_old = beta0[l-1];
       for (int j=0; j<p; j++) beta_old[j] = beta[(l-1)*p+j];
     }
+
+    // Check dfmax, gmax
+    int ng = 0;
+    int nv = 0;
+    for (int g=0; g<J; g++) {
+      if (beta_old[K1[g]] != 0) {
+	ng++;
+	nv = nv + (K1[g+1]-K1[g]);
+      }
+    }
+    if (ng > gmax | nv > dfmax) {
+      for (int ll=l; ll<L; ll++) {
+	for (int j=0; j<p; j++) beta[ll*p+j] = R_NaReal;
+      }
+      Free(beta_old);
+      Free(r);
+      Free(active);
+      return;
+    }
+
     while (iter[l] < max_iter) {
       int converged = 0;
       iter[l]++;
 
-      // Check dfmax
-      int a = 0;
-      for (int j=0; j<p; j++) a += (beta_old[j] !=0);
-      if (a > dfmax) {
-	for (int ll=l; ll<L; ll++) {
-	  for (int j=0; j<p; j++) beta[ll*p+j] = R_NaReal;
-	}
-	Free(beta_old);
-	Free(r);
-	Free(active);
-	return;
-      }
-
       // Approximate L
       Dev[l] = 0;
       for (int i=0; i<n; i++) {
 	eta = beta0_old;
 	for (int j=0; j<K0; j++) eta += x[n*j+i] * beta_old[j];
 	for (int g=0; g<J; g++) {
-	  if (active[g]) {
+	  if (beta_old[K1[g]] != 0) {
 	    for (int j=K1[g]; j<K1[g+1]; j++) eta += x[n*j+i] * beta_old[j];
 	  }
 	}
@@ -609,7 +638,7 @@ static void grPathFit_binomial(double *beta0, double *beta, int *iter, double *d
 
       // Update penalized groups
       for (int g=0; g<J; g++) {
-	if (user | l!=0) gd_binomial(beta, x, r, g, K1, active, n, l, p, penalty, lam1[l]*group_multiplier[g], lam2[l], gamma, df, beta_old);
+	if (user | l!=0) gd_binomial(beta, x, r, g, K1, n, l, p, penalty, lam1[l]*group_multiplier[g], lam2[l], gamma, df, beta_old);
       }
 
       // Check convergence
@@ -627,10 +656,10 @@ static void grPathFit_binomial(double *beta0, double *beta, int *iter, double *d
 }
 
 static const R_CMethodDef cMethods[] = {
-  {"gpPathFit_gaussian", (DL_FUNC) &gpPathFit_gaussian, 23},
-  {"gpPathFit_binomial", (DL_FUNC) &gpPathFit_binomial, 25},
-  {"grPathFit_gaussian", (DL_FUNC) &grPathFit_gaussian, 21},
-  {"grPathFit_binomial", (DL_FUNC) &grPathFit_binomial, 23},
+  {"gpPathFit_gaussian", (DL_FUNC) &gpPathFit_gaussian, 24},
+  {"gpPathFit_binomial", (DL_FUNC) &gpPathFit_binomial, 26},
+  {"grPathFit_gaussian", (DL_FUNC) &grPathFit_gaussian, 22},
+  {"grPathFit_binomial", (DL_FUNC) &grPathFit_binomial, 24},
   NULL
 };
 
